package com.jiuqi.util;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

public class StringList implements Serializable, Cloneable {
   private static final int DELIMITER_DEFINED = 1;
   private static final int QUOTECHAR_DEFINED = 2;
   private static final int NAMEVALSEP_DEFINED = 4;
   protected List items = new ArrayList();
   protected List objects = null;
   private char delimiter = 0;
   private char quoteChar = 0;
   private char nameValueSeparator = 0;
   private int defined = 0;
   private static final String CS_GB2312 = "GB2312";
   private static final String CS_ISO_8859_1 = "ISO-8859-1";

   protected boolean getDefined(int item) {
      return (this.defined & item) != 0;
   }

   protected void setDefined(int item) {
      this.defined |= item;
   }

   public boolean isEmpty() {
      return this.items.size() == 0;
   }

   public int size() {
      return this.items.size();
   }

   public void clear() {
      this.items.clear();
      if (this.objects != null) {
         this.objects.clear();
      }

   }

   public String get(int index) {
      return (String)this.items.get(index);
   }

   public void set(int index, String newValue) {
      this.items.set(index, newValue);
   }

   public Object getObject(int index) {
      return this.objects == null ? null : this.objects.get(index);
   }

   public Object getObject(String item) {
      if (this.objects == null) {
         return null;
      } else {
         int index = this.indexOf(item);
         return index < 0 ? null : this.objects.get(index);
      }
   }

   public void setObject(int index, Object value) {
      if (value != null || this.objects != null) {
         if (this.objects == null) {
            this.objects = new ArrayList(this.size());

            for(int i = 0; i < this.size(); ++i) {
               this.objects.add((Object)null);
            }
         }

         this.objects.set(index, value);
      }
   }

   public void setObject(String item, Object value) {
      int index = this.indexOf(item);
      if (index < 0) {
         this.add(item);
         index = this.size() - 1;
      }

      this.setObject(index, value);
   }

   public void assign(StringList source) {
      if (source.items == null) {
         if (this.items != null) {
            this.items.clear();
         }
      } else {
         if (this.items == null) {
            this.items = new ArrayList();
         } else {
            this.items.clear();
         }

         this.items.addAll(source.items);
      }

      if (source.objects == null) {
         if (this.objects != null) {
            this.objects.clear();
         }
      } else {
         if (this.objects == null) {
            this.objects = new ArrayList();
         } else {
            this.objects.clear();
         }

         this.objects.addAll(source.objects);
      }

   }

   public void add(String item) {
      this.items.add(item);
      if (this.objects != null) {
         this.objects.add((Object)null);
      }

   }

   public void add(String item, Object data) {
      this.items.add(item);
      if (this.objects != null) {
         this.objects.add((Object)null);
      }

      this.setObject(this.items.size() - 1, data);
   }

   public void add(int index, String item) {
      this.items.add(index, item);
      if (this.objects != null) {
         this.objects.add(index, (Object)null);
      }

   }

   public void remove(int index) {
      this.items.remove(index);
      if (this.objects != null) {
         this.objects.remove(index);
      }

   }

   public void remove(String item) {
      for(int i = this.items.size() - 1; i >= 0; --i) {
         String str = (String)this.items.get(i);
         if (str == item || str != null && str.equals(item)) {
            this.items.remove(i);
            if (this.objects != null) {
               this.objects.remove(i);
            }
            break;
         }
      }

   }

   public int indexOf(String item) {
      return this.items.indexOf(item);
   }

   public int find(String item) {
      int sb = 0;
      int se = this.items.size() - 1;
      int flag = -1;

      while(sb <= se) {
         int sp = sb + se >> 1;
         flag = compare(item, (String)this.items.get(sp));
         if (flag > 0) {
            sb = sp + 1;
         } else {
            if (flag >= 0) {
               sb = sp;
               break;
            }

            se = sp - 1;
         }
      }

      return flag == 0 ? sb : -sb - 1;
   }

   public int addsort(String item) {
      int index = this.find(item);
      if (index < 0) {
         this.items.add(-index - 1, item);
         if (this.objects != null) {
            this.objects.add(-index - 1, (Object)null);
         }
      }

      return index;
   }

   public int indexOfObject(Object obj) {
      return this.objects == null ? -1 : this.objects.indexOf(obj);
   }

   protected void quicksort(int L, int R) {
      int I;
      do {
         I = L;
         int J = R;
         String zfn = (String)this.items.get(L + R >> 1);

         do {
            while(compare(zfn, (String)this.items.get(I)) > 0) {
               ++I;
            }

            while(compare(zfn, (String)this.items.get(J)) < 0) {
               --J;
            }

            if (I <= J) {
               Object T = this.items.get(I);
               this.items.set(I, this.items.get(J));
               this.items.set(J, T);
               if (this.objects != null) {
                  T = this.objects.get(I);
                  this.objects.set(I, this.objects.get(J));
                  this.objects.set(J, T);
               }

               ++I;
               --J;
            }
         } while(I <= J);

         if (L < J) {
            this.quicksort(L, J);
         }

         L = I;
      } while(I < R);

   }

   public void sort() {
      if (this.items.size() > 1) {
         this.quicksort(0, this.items.size() - 1);
      }

   }

   public String getText() {
      StringBuffer text = new StringBuffer(1024);

      for(int i = 0; i < this.items.size(); ++i) {
         text.append(this.get(i));
         text.append('\n');
      }

      return text.toString();
   }

   public void setText(String newText) {
      StringTokenizer st = new StringTokenizer(newText, "\n");
      this.items.clear();

      String s;
      for(; st.hasMoreTokens(); this.items.add(s)) {
         s = st.nextToken();
         if (s != null && s.length() > 0 && s.charAt(s.length() - 1) == '\r') {
            s = s.substring(0, s.length() - 1);
         }
      }

      this.objects = null;
   }

   public String[] toStringArray() {
      if (this.size() == 0) {
         return null;
      } else {
         String[] result = new String[this.size()];

         for(int i = 0; i < this.size(); ++i) {
            result[i] = this.get(i);
         }

         return result;
      }
   }

   public Object[] toObjectArray() {
      if (this.size() == 0) {
         return null;
      } else {
         Object[] result = new Object[this.size()];

         for(int i = 0; i < this.size(); ++i) {
            result[i] = this.getObject(i);
         }

         return result;
      }
   }

   public String toString() {
      StringBuffer result = new StringBuffer("[");

      for(int i = 0; i < this.size(); ++i) {
         result.append("(\"" + this.get(i) + "\", ");
         result.append(this.getObject(i));
         result.append(")");
         if (i < this.size() - 1) {
            result.append(", ");
         }
      }

      result.append("]");
      return result.toString();
   }

   public Object clone() {
      StringList result = null;

      try {
         result = (StringList)super.clone();
         result.items = this.items == null ? null : (List)((ArrayList)this.items).clone();
         result.objects = this.objects == null ? null : (List)((ArrayList)this.objects).clone();
      } catch (CloneNotSupportedException var2) {
      }

      return result;
   }

   public static int compare(String s1, String s2) {
      if (s1 == null && s2 == null) {
         return 0;
      } else if (s1 == null) {
         return -1;
      } else if (s2 == null) {
         return 1;
      } else {
         try {
            String ns1 = new String(s1.toString().getBytes("GB2312"), "ISO-8859-1");
            String ns2 = new String(s2.toString().getBytes("GB2312"), "ISO-8859-1");
            return ns1.compareTo(ns2);
         } catch (Exception var4) {
            return 0;
         }
      }
   }

   public char getDelimiter() {
      return this.getDefined(1) ? this.delimiter : ';';
   }

   public void setDelimiter(char value) {
      this.delimiter = value;
      this.setDefined(1);
   }

   public char getQuoteChar() {
      return this.getDefined(2) ? this.quoteChar : '"';
   }

   public void setQuoteChar(char value) {
      this.quoteChar = value;
      this.setDefined(2);
   }

   public char getNameValueSeparator() {
      return this.getDefined(4) ? this.nameValueSeparator : '=';
   }

   public void setNameValueSeparator(char value) {
      this.nameValueSeparator = value;
      this.setDefined(4);
   }

   public String getDelimitedText() {
      StringBuffer retText = new StringBuffer();
      int count = this.size();
      if (count == 1 && StringUtils.isEmpty(this.get(0))) {
         retText.append(this.getQuoteChar());
         retText.append(this.getQuoteChar());
      } else {
         for(int i = 0; i < count; ++i) {
            String s = this.get(i);
            if (s.indexOf(this.getQuoteChar()) != -1 || s.indexOf(this.getDelimiter()) != -1) {
               s = StringUtils.quote(s, this.getQuoteChar());
            }

            if (i > 0) {
               retText.append(this.getDelimiter());
            }

            retText.append(s);
         }
      }

      return retText.toString();
   }

   public void setDelimitedText(String value) {
      this.clear();
      if (!StringUtils.isEmpty(value)) {
         StringBuffer buffer = new StringBuffer();
         boolean startQuote = false;
         int i = 0;

         while(i < value.length()) {
            char curChar = value.charAt(i++);
            if (curChar == this.getQuoteChar()) {
               if (startQuote) {
                  if (i >= value.length()) {
                     break;
                  }

                  if (value.charAt(i) == this.getQuoteChar()) {
                     buffer.append(value.charAt(i));
                     ++i;
                  } else if (value.charAt(i) == this.getDelimiter()) {
                     startQuote = false;
                  } else {
                     buffer.append(value.charAt(i));
                     ++i;
                  }
               } else if (buffer.length() > 0) {
                  buffer.append(curChar);
               } else {
                  startQuote = true;
               }
            } else if (curChar == this.getDelimiter()) {
               if (startQuote) {
                  buffer.append(curChar);
               } else {
                  this.add(buffer.toString());
                  buffer = new StringBuffer();
                  startQuote = false;
               }
            } else {
               buffer.append(curChar);
            }
         }

         this.add(buffer.toString());
      }
   }

   protected String extractName(String s) {
      if (s == null) {
         return null;
      } else {
         int index = s.indexOf(this.getNameValueSeparator());
         return index == -1 ? null : s.substring(0, index);
      }
   }

   public String getName(int index) {
      return this.extractName(this.get(index));
   }

   public int indexOfName(String name) {
      if (name == null) {
         return -1;
      } else {
         for(int i = 0; i < this.size(); ++i) {
            if (compare(name, this.getName(i)) == 0) {
               return i;
            }
         }

         return -1;
      }
   }

   public String getValue(String name) {
      int index = this.indexOfName(name);
      return index == -1 ? null : this.get(index).substring(name.length() + 1);
   }

   public void setValue(String name, String value) {
      int index = this.indexOfName(name);
      if (StringUtils.isEmpty(value)) {
         if (index != -1) {
            this.remove(index);
         }
      } else if (index == -1) {
         this.add(name + this.getNameValueSeparator() + value);
      } else {
         this.set(index, name + this.getNameValueSeparator() + value);
      }

   }

   public String getValue(int index) {
      String name = this.getName(index);
      return StringUtils.isEmpty(name) ? this.get(index) : this.get(index).substring(name.length() + 1);
   }

   public void setValue(int index, String value) {
      if (StringUtils.isEmpty(value)) {
         if (index >= 0) {
            this.remove(index);
         }
      } else if (index >= 0) {
         this.set(index, this.getName(index) + this.getNameValueSeparator() + value);
      } else {
         this.add(this.getNameValueSeparator() + value);
      }

   }
}
